@@ -56,14 +56,16 @@ import os
 
 import numpy as np
 
-from pyfits.file import FILE_MODES, _File
-from pyfits.hdu.base import _BaseHDU, _ValidHDU
-from pyfits.hdu.hdulist import fitsopen
-from pyfits.hdu.image import PrimaryHDU, ImageHDU
-from pyfits.hdu.table import BinTableHDU
-from pyfits.header import Header
-from pyfits.util import (deprecated, fileobj_closed, fileobj_name,
-                         fileobj_mode, fileobj_closed, _is_int)
+from .extern.six import string_types
+
+from .file import FILE_MODES, _File
+from .hdu.base import _BaseHDU, _ValidHDU
+from .hdu.hdulist import fitsopen
+from .hdu.image import PrimaryHDU, ImageHDU
+from .hdu.table import BinTableHDU
+from .header import Header
+from .util import (deprecated, fileobj_closed, fileobj_name, fileobj_mode,
+                   fileobj_closed, _is_int)
 
 
 __all__ = ['getheader', 'getdata', 'getval', 'setval', 'delval', 'writeto',
@@ -378,9 +380,11 @@ def writeto(filename, data, header=None, output_verify='exception',
         argument is optional.
 
     output_verify : str
-        Output verification option.  Must be one of ``"fix"``,
-        ``"silentfix"``, ``"ignore"``, ``"warn"``, or
-        ``"exception"``.  See :ref:`verify` for more info.
+        Output verification option.  Must be one of ``"fix"``, ``"silentfix"``,
+        ``"ignore"``, ``"warn"``, or ``"exception"``.  May also be any
+        combination of ``"fix"`` or ``"silentfix"`` with ``"+ignore"``,
+        ``+warn``, or ``+exception" (e.g. ``"fix+warn"``).  See :ref:`verify`
+        for more info.
 
     clobber : bool, optional
         If `True`, and if filename already exists, it will overwrite
@@ -692,7 +696,7 @@ def _getext(filename, mode, *args, **kwargs):
             if ext is not None or extname is not None or extver is not None:
                 raise TypeError(err_msg)
             ext = args[0]
-        elif isinstance(args[0], basestring):
+        elif isinstance(args[0], string_types):
             # The first arg is an extension name; it could still be valid
             # to provide an extver kwarg
             if ext is not None or extname is not None:
@@ -714,11 +718,11 @@ def _getext(filename, mode, *args, **kwargs):
     if (ext is not None and
             not (_is_int(ext) or
                  (isinstance(ext, tuple) and len(ext) == 2 and
-                  isinstance(ext[0], basestring) and _is_int(ext[1])))):
+                  isinstance(ext[0], string_types) and _is_int(ext[1])))):
         raise ValueError(
             'The ext keyword must be either an extension number '
             '(zero-indexed) or a (extname, extver) tuple.')
-    if extname is not None and not isinstance(extname, basestring):
+    if extname is not None and not isinstance(extname, string_types):
         raise ValueError('The extname argument must be a string.')
     if extver is not None and not _is_int(extver):
         raise ValueError('The extver argument must be an integer.')
