@@ -1,5 +1,6 @@
 import copy
 import operator
+import sys
 import warnings
 import weakref
 
@@ -7,11 +8,14 @@ import numpy as np
 
 from numpy import char as chararray
 
-from pyfits.column import (ASCIITNULL, FITS2NUMPY, ASCII2NUMPY, ASCII2STR,
-                           ColDefs, _AsciiColDefs, _FormatX, _FormatP, _VLF,
-                           _get_index, _wrapx, _unwrapx, _makep,
-                           _convert_ascii_format, Delayed)
-from pyfits.util import encode_ascii, decode_ascii, lazyproperty
+from .extern.six import string_types
+from .extern.six.moves import range, reduce
+
+from .column import (ASCIITNULL, FITS2NUMPY, ASCII2NUMPY, ASCII2STR, ColDefs,
+                     _AsciiColDefs, _FormatX, _FormatP, _VLF, _get_index,
+                     _wrapx, _unwrapx, _makep, _convert_ascii_format, Delayed)
+from .util import (encode_ascii, decode_ascii, lazyproperty,
+                   PyfitsDeprecationWarning)
 
 
 class FITS_record(object):
@@ -48,7 +52,7 @@ class FITS_record(object):
         for arg in [('startColumn', 'start'), ('endColumn', 'end')]:
             if arg[0] in kwargs:
                 warnings.warn('The %s argument to FITS_record is deprecated; '
-                              'use %s instead' % arg, DeprecationWarning)
+                              'use %s instead' % arg, PyfitsDeprecationWarning)
                 if arg[0] == 'startColumn':
                     start = kwargs[arg[0]]
                 elif arg[0] == 'endColumn':
@@ -66,7 +70,7 @@ class FITS_record(object):
         self.base = base
 
     def __getitem__(self, key):
-        if isinstance(key, basestring):
+        if isinstance(key, string_types):
             indx = _get_index(self.array.names, key)
 
             if indx < self.start or indx > self.end - 1:
@@ -83,13 +87,13 @@ class FITS_record(object):
         return self.array.field(indx)[self.row]
 
     def __setitem__(self, key, value):
-        if isinstance(key, basestring):
+        if isinstance(key, string_types):
             indx = _get_index(self.array._coldefs.names, key)
 
             if indx < self.start or indx > self.end - 1:
                 raise KeyError("Key '%s' does not exist." % key)
         elif isinstance(key, slice):
-            for indx in xrange(slice.start, slice.stop, slice.step):
+            for indx in range(slice.start, slice.stop, slice.step):
                 indx = self._get_indx(indx)
                 self.array.field(indx)[self.row] = value
         else:
@@ -103,7 +107,7 @@ class FITS_record(object):
         return self[slice(start, end)]
 
     def __len__(self):
-        return len(xrange(self.start, self.end, self.step))
+        return len(range(self.start, self.end, self.step))
 
     def __repr__(self):
         """
@@ -111,7 +115,7 @@ class FITS_record(object):
         """
 
         outlist = []
-        for idx in xrange(len(self)):
+        for idx in range(len(self)):
             outlist.append(repr(self[idx]))
         return '(%s)' % ', '.join(outlist)
 
@@ -181,7 +185,6 @@ class FITS_rec(np.recarray):
         self._coldefs = None
         self._gap = 0
         self._uint = False
-        self.names = list(self.dtype.names)
         self.formats = None
         return self
 
@@ -210,8 +213,7 @@ class FITS_rec(np.recarray):
         meta = []
 
         for attrs in ['_convert', '_heapoffset', '_heapsize', '_nfields',
-                      '_gap', '_uint', 'names', 'formats', 'parnames',
-                      '_coldefs']:
+                      '_gap', '_uint', 'formats', 'parnames', '_coldefs']:
 
             try:
                 # _coldefs can be Delayed, and file objects cannot be
@@ -240,7 +242,6 @@ class FITS_rec(np.recarray):
             self._nfields = obj._nfields
             self._gap = obj._gap
             self._uint = obj._uint
-            self.names = obj.names
             self.formats = obj.formats
         else:
             # This will allow regular ndarrays with fields, rather than
@@ -256,7 +257,6 @@ class FITS_rec(np.recarray):
             self._uint = getattr(obj, '_uint', False)
 
             # Bypass setattr-based assignment to fields; see #86
-            self.names = list(obj.dtype.names)
             self.formats = None
 
             attrs = ['_convert', '_coldefs', '_gap']
@@ -478,7 +478,7 @@ class FITS_rec(np.recarray):
             return super(FITS_rec, self).__setattr__(attr, value)
 
     def __getitem__(self, key):
-        if isinstance(key, basestring):
+        if isinstance(key, string_types):
             return self.field(key)
         elif isinstance(key, (slice, np.ndarray, tuple, list)):
             # Have to view as a recarray then back as a FITS_rec, otherwise the
@@ -519,7 +519,7 @@ class FITS_rec(np.recarray):
             return newrecord
 
     def __setitem__(self, key, value):
-        if isinstance(key, basestring):
+        if isinstance(key, string_types):
             self[key][:] = value
             return
 
@@ -580,18 +580,40 @@ class FITS_rec(np.recarray):
         """
         A user-visible accessor for the coldefs.
 
-        See https://trac.assembla.com/pyfits/ticket/44
+        See https://aeon.stsci.edu/ssb/trac/pyfits/ticket/44
         """
 
         return self._coldefs
 
+    @property
+    def names(self):
+        """List of column names."""
+
+        if hasattr(self, '_coldefs') and self._coldefs is not None:
+            return self._coldefs.names
+        else:
+            return list(self.dtype.names)
+
+
     def field(self, key):
         """
         A view of a `Column`'s data as an array.
         """
 
-        indx = _get_index(self.names, key)
-        recformat = self._coldefs._recformats[indx]
+        # NOTE: The *column* index may not be the same as the field index in
+        # the recarray, if the column is a phantom column
+        col_indx = _get_index(self.columns.names, key)
+        if self.columns[col_indx]._phantom:
+            warnings.warn(
+                'Field %r has a repeat count of 0 in its format code, '
+                'indicating an empty field.' % key)
+            recformat = self.columns._recformats[col_indx].lstrip('0')
+            return np.array([], dtype=recformat)
+        # Ignore phantom columns in determining the physical field number
+        n_phantom = len([c for c in self.columns[:col_indx] if c._phantom])
+        field_indx = col_indx - n_phantom
+
+        recformat = self._coldefs._recformats[col_indx]
 
         # If field's base is a FITS_rec, we can run into trouble because it
         # contains a reference to the ._coldefs object of the original data;
@@ -603,21 +625,21 @@ class FITS_rec(np.recarray):
         # base could still be a FITS_rec in some cases, so take care to
         # use rec.recarray.field to avoid a potential infinite
         # recursion
-        field = np.recarray.field(base, indx)
+        field = np.recarray.field(base, field_indx)
 
-        if self._convert[indx] is None:
+        if self._convert[field_indx] is None:
             if isinstance(recformat, _FormatP):
                 # for P format
-                converted = self._convert_p(indx, field, recformat)
+                converted = self._convert_p(col_indx, field, recformat)
             else:
                 # Handle all other column data types which are fixed-width
                 # fields
-                converted = self._convert_other(indx, field, recformat)
+                converted = self._convert_other(col_indx, field, recformat)
 
-            self._convert[indx] = converted
+            self._convert[field_indx] = converted
             return converted
 
-        return self._convert[indx]
+        return self._convert[field_indx]
 
     def _convert_x(self, field, recformat):
         """Convert a raw table column to a bit array as specified by the
@@ -639,9 +661,9 @@ class FITS_rec(np.recarray):
         if raw_data is None:
             raise IOError(
                 "Could not find heap data for the %r variable-length "
-                "array column." % self.names[indx])
+                "array column." % self.columns.names[indx])
 
-        for idx in xrange(len(self)):
+        for idx in range(len(self)):
             offset = field[idx, 1] + self._heapoffset
             count = field[idx, 0]
 
@@ -689,10 +711,12 @@ class FITS_rec(np.recarray):
 
         try:
             dummy = np.array(dummy, dtype=recformat)
-        except ValueError, e:
+        except ValueError:
+            exc = sys.exc_info()[1]
             raise ValueError(
                 '%s; the header may be missing the necessary TNULL%d '
-                'keyword or the table contains invalid data' % (e, indx + 1))
+                'keyword or the table contains invalid data' %
+                (exc, indx + 1))
 
         return dummy
 
@@ -785,7 +809,7 @@ class FITS_rec(np.recarray):
                     except OverflowError:
                         warnings.warn(
                             "Overflow detected while applying TZERO{0:d}. "
-                            "Returning unscaled data.".format(indx))
+                            "Returning unscaled data.".format(indx + 1))
                     else:
                         field = test_overflow
                 else:
@@ -822,6 +846,20 @@ class FITS_rec(np.recarray):
         hdu = new_table(self._coldefs, nrows=shape[0])
         return hdu.data
 
+    def _get_heap_data(self):
+        """
+        Returns a pointer into the table's raw data to its heap (if present).
+
+        This is returned as a numpy byte array.
+        """
+
+        if self._heapsize:
+            raw_data = self._get_raw_data().view(np.ubyte)
+            heap_end = self._heapoffset + self._heapsize
+            return raw_data[self._heapoffset:heap_end]
+        else:
+            return np.array([], dtype=np.ubyte)
+
     def _get_raw_data(self):
         """
         Returns the base array of self that "raw data array" that is the
@@ -877,9 +915,16 @@ class FITS_rec(np.recarray):
 
         return (_str, _bool, _number, _scale, _zero, bscale, bzero, dim)
 
-    def _scale_back(self):
+    def _scale_back(self, update_heap_pointers=True):
         """
         Update the parent array, using the (latest) scaled array.
+
+        If ``update_heap_pointers`` is `False`, this will leave all the heap
+        pointers in P/Q columns as they are verbatim--it only makes sense to do
+        this if there is already data on the heap and it can be guaranteed that
+        that data has not been modified, and there is not new data to add to
+        the heap.  Currently this is only used as an optimization for
+        CompImageHDU that does its own handling of the heap.
         """
 
         # Running total for the new heap size
@@ -898,7 +943,7 @@ class FITS_rec(np.recarray):
                 # an array of characters.
                 dtype = np.array([], dtype=recformat.dtype).dtype
 
-                if self._convert[indx] is not None:
+                if update_heap_pointers and self._convert[indx] is not None:
                     # The VLA has potentially been updated, so we need to
                     # update the array descriptors
                     field[:] = 0  # reset
@@ -910,8 +955,9 @@ class FITS_rec(np.recarray):
                     field[:, 1][:] += heapsize
 
                 heapsize += field[:, 0].sum() * dtype.itemsize
-                # Even if this VLA has not been read, we need to include
-                # the size of its constituent arrays in the heap size total
+                # Even if this VLA has not been read or updated, we need to
+                # include the size of its constituent arrays in the heap size
+                # total
 
             if self._convert[indx] is None:
                 continue
@@ -1010,7 +1056,7 @@ class FITS_rec(np.recarray):
                         else:
                             pad = self._coldefs._padding_byte.encode('ascii')
 
-                        for idx in xrange(len(dummy)):
+                        for idx in range(len(dummy)):
                             val = dummy[idx]
                             dummy[idx] = val + (pad * (itemsize - len(val)))
 
