@@ -6,16 +6,19 @@ import warnings
 
 import numpy as np
 
-from pyfits.card import Card
-from pyfits.column import Column, ColDefs, _FormatP, TDEF_RE
-from pyfits.column import KEYWORD_NAMES as TABLE_KEYWORD_NAMES
-from pyfits.fitsrec import FITS_rec
-from pyfits.hdu.base import DELAYED, ExtensionHDU
-from pyfits.hdu.image import _ImageBaseHDU, ImageHDU
-from pyfits.hdu.table import BinTableHDU
-from pyfits.header import Header
-from pyfits.util import (lazyproperty, _is_pseudo_unsigned, _unsigned_zero,
-                         deprecated, _is_int)
+from ..extern.six import string_types, iteritems
+from ..extern.six.moves import range
+
+from ..card import Card
+from ..column import Column, ColDefs, _FormatP, TDEF_RE
+from ..column import KEYWORD_NAMES as TABLE_KEYWORD_NAMES
+from ..fitsrec import FITS_rec
+from ..header import Header
+from ..util import (lazyproperty, _is_pseudo_unsigned, _unsigned_zero,
+                    deprecated, _is_int, PyfitsPendingDeprecationWarning)
+from .base import DELAYED, ExtensionHDU
+from .image import _ImageBaseHDU, ImageHDU
+from .table import BinTableHDU
 
 try:
     from pyfits import compression
@@ -163,7 +166,7 @@ class CompImageHeader(Header):
     def append(self, card=None, useblanks=True, bottom=False, end=False):
         # This logic unfortunately needs to be duplicated from the base class
         # in order to determine the keyword
-        if isinstance(card, basestring):
+        if isinstance(card, string_types):
             card = Card(card)
         elif isinstance(card, tuple):
             card = Card(*card)
@@ -198,7 +201,7 @@ class CompImageHeader(Header):
                 self.append(card, end=True)
                 return
 
-        if isinstance(card, basestring):
+        if isinstance(card, string_types):
             card = Card(card)
         elif isinstance(card, tuple):
             card = Card(*card)
@@ -353,6 +356,8 @@ class CompImageHeader(Header):
         return idx
 
 
+# TODO: Fix this class so that it doesn't actually inherit from BinTableHDU,
+# but instead has an internal BinTableHDU reference
 class CompImageHDU(BinTableHDU):
     """
     Compressed Image HDU class.
@@ -365,6 +370,15 @@ class CompImageHDU(BinTableHDU):
         'quantizeLevel': 'quantize_level'
     }
 
+    _manages_own_heap = True
+    """
+    The calls to CFITSIO lay out the heap data in memory, and we write it out
+    the same way CFITSIO organizes it.  In principle this would break if a user
+    manually changes the underlying compressed data by hand, but there is no
+    reason they would want to do that (and if they do that's their
+    responsibility).
+    """
+
     def __init__(self, data=None, header=None, name=None,
                  compression_type=DEFAULT_COMPRESSION_TYPE,
                  tile_size=None,
@@ -588,7 +602,7 @@ class CompImageHDU(BinTableHDU):
                 warnings.warn('Keyword argument %s to %s is pending '
                               'deprecation; use %s instead' %
                               (oldarg, self.__class__.__name__, newarg),
-                              PendingDeprecationWarning)
+                              PyfitsPendingDeprecationWarning)
                 compression_opts[newarg] = kwargs[oldarg]
                 del kwargs[oldarg]
             else:
@@ -624,7 +638,7 @@ class CompImageHDU(BinTableHDU):
         self._scale_back = scale_back
 
         self._axes = [self._header.get('ZNAXIS' + str(axis + 1), 0)
-                      for axis in xrange(self._header.get('ZNAXIS', 0))]
+                      for axis in range(self._header.get('ZNAXIS', 0))]
 
         # store any scale factors from the table header
         if do_not_scale_image_data:
@@ -646,7 +660,7 @@ class CompImageHDU(BinTableHDU):
             return False
 
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
 
         if xtension not in ('BINTABLE', 'A3DTABLE'):
@@ -744,13 +758,16 @@ class CompImageHDU(BinTableHDU):
         # heuristic used by CFITSIO, so this should give consistent results.
         # And the cases where this heuristic is insufficient are extreme and
         # almost entirely contrived corner cases, so it will do for now
-        huge_hdu = self.data.nbytes > 2 ** 32
+        if self._has_data:
+            huge_hdu = self.data.nbytes > 2 ** 32
 
-        if huge_hdu and not CFITSIO_SUPPORTS_Q_FORMAT:
-            raise IOError(
-                "PyFITS cannot compress images greater than 4 GB in size "
-                "(%s is %s bytes) without CFITSIO >= 3.35" %
-                ((self.name, self.ver), self.data.nbytes))
+            if huge_hdu and not CFITSIO_SUPPORTS_Q_FORMAT:
+                raise IOError(
+                    "PyFITS cannot compress images greater than 4 GB in size "
+                    "(%s is %s bytes) without CFITSIO >= 3.35" %
+                    ((self.name, self.ver), self.data.nbytes))
+        else:
+            huge_hdu = False
 
         # Update the extension name in the table header
         if not name and not 'EXTNAME' in self._header:
@@ -961,7 +978,7 @@ class CompImageHDU(BinTableHDU):
                     # user specified tile size is too small
                     raise ValueError('Hcompress minimum tile dimension is '
                                      '4 pixels')
-                major_dims = len(filter(lambda x: x > 1, tile_size))
+                major_dims = len([ts for ts in tile_size if ts > 1])
                 if major_dims > 2:
                     raise ValueError(
                         'HCOMPRESS can only support 2-dimensional tile sizes.'
@@ -1043,7 +1060,7 @@ class CompImageHDU(BinTableHDU):
 
         # Calculate the number of rows in the output table and
         # write the ZNAXISn and ZTILEn cards to the table header.
-        nrows = 1
+        nrows = 0
 
         for idx, axis in enumerate(self._axes):
             naxis = 'NAXIS' + str(idx + 1)
@@ -1063,7 +1080,10 @@ class CompImageHDU(BinTableHDU):
                     ts = self._header[ztile]
                 tile_size.append(ts)
 
-            nrows = nrows * ((axis - 1) // ts + 1)
+            if not nrows:
+                nrows = (axis - 1) // ts + 1
+            else:
+                nrows *= ((axis - 1) // ts + 1)
 
             if image_header and naxis in image_header:
                 self._header.set(znaxis, axis, image_header.comments[naxis],
@@ -1201,8 +1221,8 @@ class CompImageHDU(BinTableHDU):
                 # no dithering, rather than whatever DEFAULT_QUANTIZE_METHOD
                 # is set to
                 quantize_method = self._header.get('ZQUANTIZ', NO_DITHER)
-                if isinstance(quantize_method, basestring):
-                    for k, v in QUANTIZE_METHOD_NAMES.items():
+                if isinstance(quantize_method, string_types):
+                    for k, v in iteritems(QUANTIZE_METHOD_NAMES):
                         if v.upper() == quantize_method:
                             quantize_method = k
                             break
@@ -1346,6 +1366,9 @@ class CompImageHDU(BinTableHDU):
         # The data attribute is the image data (not the table data).
         data = compression.decompress_hdu(self)
 
+        if data is None:
+            return data
+
         # Scale the data if necessary
         if (self._orig_bzero != 0 or self._orig_bscale != 1):
             new_dtype = self._dtype_for_bitpix()
@@ -1627,69 +1650,15 @@ class CompImageHDU(BinTableHDU):
         self.compressed_data._heapsize = heapsize
         self.compressed_data.formats = self.columns.formats
 
-        # Update the table header cards to match the compressed data.
-        self._update_header()
-
-    @deprecated('3.2', alternative='(refactor your code)', pending=True)
+    @deprecated('3.2', alternative='(refactor your code)')
     def updateCompressedData(self):
         self._update_compressed_data()
 
-    def _update_header(self):
-        """
-        Update the table header cards to match the compressed data.
-        """
-
-        # Get the _heapsize attribute to match the data.
-        self.compressed_data._scale_back()
-
-        # Check that TFIELDS and NAXIS2 match the data.
-        self._header['TFIELDS'] = self.compressed_data._nfields
-        self._header['NAXIS2'] = self.compressed_data.shape[0]
-
-        # Calculate PCOUNT, for variable length tables.
-        _tbsize = self._header['NAXIS1'] * self._header['NAXIS2']
-        _heapstart = self._header.get('THEAP', _tbsize)
-        self.compressed_data._gap = _heapstart - _tbsize
-        _pcount = self.compressed_data._heapsize + self.compressed_data._gap
-
-        if _pcount > 0:
-            self._header['PCOUNT'] = _pcount
-
-        # Update TFORM for variable length columns.
-        for idx in range(self.compressed_data._nfields):
-            format = self.compressed_data._coldefs._recformats[idx]
-            if isinstance(format, _FormatP):
-                _max = self.compressed_data.field(idx).max
-                format_cls = format.__class__
-                format = format_cls(format.dtype, repeat=format.repeat,
-                                    max=_max)
-                self._header['TFORM' + str(idx + 1)] = format.tform
-        # Insure that for RICE_1 that the BLOCKSIZE and BYTEPIX cards
-        # are present and set to the hard coded values used by the
-        # compression algorithm.
-        if self._header['ZCMPTYPE'] == 'RICE_1':
-            self._header.set('ZNAME1', 'BLOCKSIZE', 'compression block size',
-                             after='ZCMPTYPE')
-            self._header.set('ZVAL1', DEFAULT_BLOCK_SIZE, 'pixels per block',
-                             after='ZNAME1')
-
-            self._header.set('ZNAME2', 'BYTEPIX',
-                             'bytes per pixel (1, 2, 4, or 8)', after='ZVAL1')
-
-            if self._header['ZBITPIX'] == 8:
-                bytepix = 1
-            elif self._header['ZBITPIX'] == 16:
-                bytepix = 2
-            else:
-                bytepix = DEFAULT_BYTE_PIX
-
-            self._header.set('ZVAL2', bytepix,
-                             'bytes per pixel (1, 2, 4, or 8)',
-                             after='ZNAME2')
-
-    @deprecated('3.2', alternative='(refactor your code)', pending=True)
+    @deprecated('3.2',
+                alternative='(refactor your code; this function no '
+                            'longer does anything)')
     def updateHeader(self):
-        self._update_header()
+        pass
 
     def scale(self, type=None, option='old', bscale=1, bzero=0):
         """
@@ -1799,8 +1768,6 @@ class CompImageHDU(BinTableHDU):
         self._orig_bzero = self._bzero
         self._orig_bscale = self._bscale
 
-    # TODO: Fix this class so that it doesn't actually inherit from
-    # BinTableHDU, but instead has an internal BinTableHDU reference
     def _prewriteto(self, checksum=False, inplace=False):
         if self._scale_back:
             self.scale(_ImageBaseHDU.NumCode[self._orig_bitpix])
@@ -1834,10 +1801,8 @@ class CompImageHDU(BinTableHDU):
             # handles it propertly
             self.__dict__['data'] = self.compressed_data
 
-        # Doesn't call the super's _prewriteto, since it calls
-        # self.data._scale_back(), which is meaningless here.
-        return ExtensionHDU._prewriteto(self, checksum=checksum,
-                                        inplace=inplace)
+        return super(CompImageHDU, self)._prewriteto(checksum=checksum,
+                                                     inplace=inplace)
 
     def _writeheader(self, fileobj):
         """
