@@ -1,12 +1,11 @@
 import sys
 import numpy as np
 
-from pyfits.column import Column, ColDefs, FITS2NUMPY
-from pyfits.fitsrec import FITS_rec, FITS_record
-from pyfits.hdu.image import _ImageBaseHDU, PrimaryHDU
-from pyfits.hdu.table import _TableLikeHDU
-from pyfits.util import (lazyproperty, _is_int, _is_pseudo_unsigned,
-                         _unsigned_zero)
+from ..column import Column, ColDefs, FITS2NUMPY
+from ..fitsrec import FITS_rec, FITS_record
+from ..util import lazyproperty, _is_int, _is_pseudo_unsigned, _unsigned_zero
+from .image import _ImageBaseHDU, PrimaryHDU
+from .table import _TableLikeHDU
 
 
 class Group(FITS_record):
@@ -201,6 +200,11 @@ class GroupData(FITS_rec):
 
     @property
     def data(self):
+        """
+        The raw group data represented as a multi-dimensional `numpy.ndarray`
+        array.
+        """
+
         # The last column in the coldefs is the data portion of the group
         return self.field(self._coldefs.names[-1])
 
@@ -505,7 +509,7 @@ class GroupsHDU(PrimaryHDU, _TableLikeHDU):
                 byteswapped = False
                 d = self.data
 
-            cs = self._compute_checksum(np.fromstring(d, dtype='ubyte'),
+            cs = self._compute_checksum(d.flatten().view(np.uint8),
                                         blocking=blocking)
 
             # If the data was byteswapped in this method then return it to
