@@ -10,19 +10,19 @@ import warnings
 import numpy as np
 from numpy import char as chararray
 
+from ..extern.six import string_types
+
 # This module may have many dependencies on pyfits.column, but pyfits.column
 # has fewer dependencies overall, so it's easier to keep table/column-related
 # utilities in pyfits.column
-from pyfits.column import (FITS2NUMPY, KEYWORD_NAMES, KEYWORD_ATTRIBUTES,
-                           TDEF_RE, Column, ColDefs, _AsciiColDefs,
-                           _FormatP, _FormatQ, _makep, _VLF, _parse_tformat,
-                           _scalar_to_format, _convert_format,
-                           _cmp_recformats, _get_index)
-from pyfits.fitsrec import FITS_rec
-from pyfits.hdu.base import DELAYED, _ValidHDU, ExtensionHDU
-from pyfits.header import Header
-from pyfits.util import (lazyproperty, _is_int, _str_to_num, _pad_length,
-                         deprecated)
+from ..column import (FITS2NUMPY, KEYWORD_NAMES, KEYWORD_ATTRIBUTES, TDEF_RE,
+                      Column, ColDefs, _AsciiColDefs, _FormatP, _FormatQ,
+                      _makep, _VLF, _parse_tformat, _scalar_to_format,
+                      _convert_format, _cmp_recformats, _get_index)
+from ..fitsrec import FITS_rec
+from ..header import Header
+from ..util import lazyproperty, _is_int, _str_to_num, _pad_length, deprecated
+from .base import DELAYED, _ValidHDU, ExtensionHDU
 
 
 class FITSTableDumpDialect(csv.excel):
@@ -61,6 +61,61 @@ class _TableLikeHDU(_ValidHDU):
 
         raise NotImplementedError
 
+    @classmethod
+    def from_columns(cls, columns, header=None, nrows=0, fill=False,
+                     **kwargs):
+        """
+        Given either a `ColDefs` object, a sequence of `Column` objects,
+        or another table HDU or table data (a `FITS_rec` or multi-field
+        `numpy.ndarray` or `numpy.recarray` object, return a new table HDU of
+        the class this method was called on using the column definition from
+        the input.
+
+        This is an alternative to the now deprecated `new_table` function,
+        and otherwise accepts the same arguments.  See also
+        `FITS_rec.from_columns`.
+
+        Parameters
+        ----------
+        columns : sequence of `Column`, `ColDefs`, or other
+            The columns from which to create the table data, or an object with
+            a column-like structure from which a `ColDefs` can be instantiated.
+            This includes an existing `BinTableHDU` or `TableHDU`, or a
+            `numpy.recarray` to give some examples.
+
+            If these columns have data arrays attached that data may be used in
+            initializing the new table.  Otherwise the input columns will be
+            used as a template for a new table with the requested number of
+            rows.
+
+        header : `Header`
+            An optional `Header` object to instantiate the new HDU yet.  Header
+            keywords specifically related to defining the table structure (such
+            as the "TXXXn" keywords like TTYPEn) will be overridden by the
+            supplied column definitions, but all other informational and data
+            model-specific keywords are kept.
+
+        nrows : int
+            Number of rows in the new table.  If the input columns have data
+            associated with them, the size of the largest input column is used.
+            Otherwise the default is 0.
+
+        fill : bool
+            If `True`, will fill all cells with zeros or blanks.  If `False`,
+            copy the data from input, undefined cells will still be filled with
+            zeros/blanks.
+
+        Notes
+        -----
+
+        Any additional keyword arguments accepted by the HDU class's
+        ``__init__`` may also be passed in as keyword arguments.
+        """
+
+        coldefs = cls._columns_type(columns)
+        data = FITS_rec.from_columns(coldefs, nrows=nrows, fill=fill)
+        return cls(data=data, header=header, **kwargs)
+
     @lazyproperty
     def columns(self):
         """
@@ -127,6 +182,16 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
     FITS table extension base HDU class.
     """
 
+    _manages_own_heap = False
+    """
+    This flag implies that when writing VLA tables (P/Q format) the heap
+    pointers that go into P/Q table columns should not be reordered or
+    rearranged in any way by the default heap management code.
+
+    This is included primarily as an optimization for compressed image HDUs
+    which perform their own heap maintenance.
+    """
+
     def __init__(self, data=None, header=None, name=None, uint=False):
         """
         Parameters
@@ -241,7 +306,7 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                     # Delete the _arrays attribute so that it is recreated to
                     # point to the new data placed in the column objects above
                     del self.columns._arrays
-                except (TypeError, AttributeError), e:
+                except (TypeError, AttributeError):
                     # This shouldn't happen as long as self.columns._arrays
                     # is a lazyproperty
                     pass
@@ -250,7 +315,7 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
             else:
                 raise TypeError('Table data has incorrect type.')
 
-        if not (isinstance(self._header[0], basestring) and
+        if not (isinstance(self._header[0], string_types) and
                 self._header[0].rstrip() == self._extension):
             self._header[0] = (self._extension, self._ext_comment)
 
@@ -335,7 +400,7 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
                 # Delete the _arrays attribute so that it is recreated to
                 # point to the new data placed in the column objects above
                 del self.columns._arrays
-            except (TypeError, AttributeError), e:
+            except (TypeError, AttributeError):
                 # This shouldn't happen as long as self.columns._arrays
                 # is a lazyproperty
                 pass
@@ -376,23 +441,23 @@ class _TableBaseHDU(ExtensionHDU, _TableLikeHDU):
         # touch the data, so it's defined (in the case of reading from a
         # FITS file)
         self.data
-        return new_table(self.columns, header=self._header,
-                         tbtype=self.columns._tbtype)
+        return new_table(self.columns, header=self._header)
 
     def _prewriteto(self, checksum=False, inplace=False):
         if self._has_data:
-            self.data._scale_back()
+            self.data._scale_back(
+                update_heap_pointers=not self._manages_own_heap)
             # check TFIELDS and NAXIS2
             self._header['TFIELDS'] = len(self.data._coldefs)
             self._header['NAXIS2'] = self.data.shape[0]
 
             # calculate PCOUNT, for variable length tables
-            tbsize = self.header['NAXIS1'] * self.header['NAXIS2']
-            heapstart = self.header.get('THEAP', tbsize)
+            tbsize = self._header['NAXIS1'] * self._header['NAXIS2']
+            heapstart = self._header.get('THEAP', tbsize)
             self.data._gap = heapstart - tbsize
             pcount = self.data._heapsize + self.data._gap
             if pcount > 0:
-                self.header['PCOUNT'] = pcount
+                self._header['PCOUNT'] = pcount
 
             # update the other T****n keywords
             self._populate_table_keywords()
@@ -504,7 +569,7 @@ class TableHDU(_TableBaseHDU):
     def match_header(cls, header):
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         return card.keyword == 'XTENSION' and xtension == cls._extension
 
@@ -586,11 +651,12 @@ class BinTableHDU(_TableBaseHDU):
     _extension = 'BINTABLE'
     _ext_comment = 'binary table extension'
 
+
     @classmethod
     def match_header(cls, header):
         card = header.cards[0]
         xtension = card.value
-        if isinstance(xtension, basestring):
+        if isinstance(xtension, string_types):
             xtension = xtension.rstrip()
         return (card.keyword == 'XTENSION' and
                 xtension in (cls._extension, 'A3DTABLE'))
@@ -654,17 +720,28 @@ class BinTableHDU(_TableBaseHDU):
 
                 nbytes = self.data._gap
 
-                for idx in range(self.data._nfields):
-                    if not isinstance(self.data.columns._recformats[idx],
-                                      _FormatP):
-                        continue
+                if not self._manages_own_heap:
+                    # Write the heap data one column at a time, in the order
+                    # that the data pointers appear in the column (regardless
+                    # if that data pointer has a different, previous heap
+                    # offset listed)
+                    for idx in range(self.data._nfields):
+                        if not isinstance(self.data.columns._recformats[idx],
+                                          _FormatP):
+                            continue
 
-                    field = self.data.field(idx)
-                    for row in field:
-                        if len(row) > 0:
-                            nbytes += row.nbytes
-                            if not fileobj.simulateonly:
-                                fileobj.writearray(row)
+                        field = self.data.field(idx)
+                        for row in field:
+                            if len(row) > 0:
+                                nbytes += row.nbytes
+                                if not fileobj.simulateonly:
+                                    fileobj.writearray(row)
+                else:
+                    heap_data = self.data._get_heap_data()
+                    if len(heap_data) > 0:
+                        nbytes += len(heap_data)
+                        if not fileobj.simulateonly:
+                            fileobj.writearray(heap_data)
 
                 self.data._heapsize = nbytes - self.data._gap
                 size += nbytes
@@ -806,7 +883,7 @@ class BinTableHDU(_TableBaseHDU):
         files = [datafile, cdfile, hfile]
 
         for f in files:
-            if isinstance(f, basestring):
+            if isinstance(f, string_types):
                 if os.path.exists(f) and os.path.getsize(f) != 0:
                     if clobber:
                         warnings.warn("Overwriting existing file %r." % f)
@@ -935,7 +1012,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'w')
             close_file = True
 
@@ -1004,7 +1081,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'w')
             close_file = True
 
@@ -1029,7 +1106,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'r')
             close_file = True
 
@@ -1095,8 +1172,8 @@ class BinTableHDU(_TableBaseHDU):
         # can convert from this format to an actual FITS file on disk without
         # needing enough physical memory to hold the entire thing at once;
         # new_table() could use a similar feature.
-        hdu = new_table(np.recarray(shape=1, dtype=dtype), nrows=nrows,
-                        fill=True)
+        hdu = BinTableHDU.from_columns(np.recarray(shape=1, dtype=dtype),
+                                       nrows=nrows, fill=True)
         data = hdu.data
         for idx, length in enumerate(vla_lengths):
             if length is not None:
@@ -1169,7 +1246,7 @@ class BinTableHDU(_TableBaseHDU):
 
         close_file = False
 
-        if isinstance(fileobj, basestring):
+        if isinstance(fileobj, string_types):
             fileobj = open(fileobj, 'r')
             close_file = True
 
@@ -1194,12 +1271,10 @@ class BinTableHDU(_TableBaseHDU):
         return ColDefs(columns)
 
 
-@deprecated('3.2',
-            alternative=':meth:`FITS_rec.from_columns` to create a new '
-                        ':class:`FITS_rec` data object from the input '
-                        'columns to pass into the constructor for '
-                        ':class:`BinTableHDU` or :class:`TableHDU`',
-            pending=True)
+@deprecated('3.3',
+            alternative=':meth:`BinTableHDU.from_columns` for new BINARY '
+                        'tables or :meth:`TableHDU.from_columns` for new '
+                        'ASCII tables')
 def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
     """
     Create a new table from the input column definitions.
@@ -1236,7 +1311,7 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
 
     # tbtype defaults to classes now, but in all prior version of PyFITS it was
     # a string, so we still support that use case as well
-    if not isinstance(tbtype, basestring):
+    if not isinstance(tbtype, string_types):
         cls = tbtype
         tbtype = cls.__name__
     else:
@@ -1246,25 +1321,12 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype=BinTableHDU):
         # hope is to deprecate this function anyways so there's not much point
         # in trying to make it more "generic".
         if tbtype.lower() == 'tablehdu':
-            tbtype = 'TableHDU'
             cls = TableHDU
         elif tbtype.lower() == 'bintablehdu':
-            tbtype = 'BinTableHDU'
             cls = BinTableHDU
         else:
             raise ValueError("tbtype must be one of 'TableHDU' or "
                              "'BinTableHDU'")
 
-    if isinstance(input, FITS_rec):  # input is a FITS_rec
-        # Create a new ColDefs object from the input FITS_rec's ColDefs
-        # object and assign it to the ColDefs attribute of the new hdu.
-        columns = ColDefs(input._coldefs, tbtype)
-    else:  # input is a list of Columns or possibly a recarray
-        # Create a new ColDefs object from the input list of Columns and
-        # assign it to the ColDefs attribute of the new hdu.
-        columns = ColDefs(input, tbtype)
-
-    data = FITS_rec.from_columns(columns, nrows=nrows, fill=fill)
-
     # construct a table HDU of the requested type
-    return cls(header=header, data=data)
+    return cls.from_columns(input, header=header, nrows=nrows, fill=fill)
