@@ -6,9 +6,9 @@ import sys
 import numpy as np
 
 import pyfits as fits
-from pyfits.hdu.base import _ValidHDU
-from pyfits.tests import PyfitsTestCase
-from pyfits.tests.test_table import comparerecords
+from ..hdu.base import _ValidHDU
+from . import PyfitsTestCase
+from .test_table import comparerecords
 
 
 class TestChecksumFunctions(PyfitsTestCase):
@@ -124,7 +124,7 @@ class TestChecksumFunctions(PyfitsTestCase):
         col1 = fits.Column(name='target', format='20A', array=a1)
         col2 = fits.Column(name='V_mag', format='E', array=a2)
         cols = fits.ColDefs([col1, col2])
-        tbhdu = fits.new_table(cols)
+        tbhdu = fits.BinTableHDU.from_columns(cols)
         tbhdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
         with fits.open(self.temp('tmp.fits'), checksum=True) as hdul:
             assert comparerecords(tbhdu.data, hdul[1].data)
@@ -141,7 +141,7 @@ class TestChecksumFunctions(PyfitsTestCase):
         c1 = fits.Column(name='var', format='PJ()',
                          array=np.array([[45.0, 56], np.array([11, 12, 13])], 'O'))
         c2 = fits.Column(name='xyz', format='2I', array=[[11, 3], [12, 4]])
-        tbhdu = fits.new_table([c1, c2])
+        tbhdu = fits.BinTableHDU.from_columns([c1, c2])
         tbhdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
         with fits.open(self.temp('tmp.fits'), checksum=True) as hdul:
             assert comparerecords(tbhdu.data, hdul[1].data)
@@ -163,8 +163,8 @@ class TestChecksumFunctions(PyfitsTestCase):
         c2 = fits.Column(name='def', format='D', array=r1, bscale=2.3,
                          bzero=0.6)
         c3 = fits.Column(name='t1', format='I', array=[91, 92, 93])
-        x = fits.ColDefs([c1, c2, c3], tbtype='TableHDU')
-        hdu = fits.new_table(x, tbtype='TableHDU')
+        x = fits.ColDefs([c1, c2, c3])
+        hdu = fits.TableHDU.from_columns(x)
         hdu.writeto(self.temp('tmp.fits'), clobber=True, checksum=True)
         with fits.open(self.temp('tmp.fits'), checksum=True) as hdul:
             assert comparerecords(hdu.data, hdul[1].data)
@@ -172,13 +172,10 @@ class TestChecksumFunctions(PyfitsTestCase):
             assert hdul[0].header['CHECKSUM'] == 'D8iBD6ZAD6fAD6ZA'
             assert 'DATASUM' in hdul[0].header
             assert hdul[0].header['DATASUM'] == '0'
+            assert 'CHECKSUM' in hdul[1].header
+            assert 'DATASUM' in hdul[1].header
             if sys.platform != 'win32':
-                # The checksum ends up being different on Windows, possibly due
-                # to slight floating point differences
-                # TODO: In Astropy mark these properly as known fail
-                assert 'CHECKSUM' in hdul[1].header
                 assert hdul[1].header['CHECKSUM'] == '51IDA1G981GCA1G9'
-                assert 'DATASUM' in hdul[1].header
                 assert hdul[1].header['DATASUM'] == '1948208413'
 
     def test_compressed_image_data(self):
@@ -342,7 +339,7 @@ class TestChecksumFunctions(PyfitsTestCase):
 
     def test_open_update_mode_preserve_checksum(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/148 where
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/148 where
         checksums are being removed from headers when a file is opened in
         update mode, even though no changes were made to the file.
         """
@@ -362,7 +359,7 @@ class TestChecksumFunctions(PyfitsTestCase):
 
     def test_open_update_mode_update_checksum(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/148, part
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/148, part
         2.  This ensures that if a file contains a checksum, the checksum is
         updated when changes are saved to the file, even if the file was opened
         with the default of checksum=False.
