@@ -6,16 +6,18 @@ import sys
 
 import numpy as np
 
+from ..extern.six import BytesIO
+
 import pyfits as fits
-from pyfits.util import BytesIO
-from pyfits.verify import VerifyError
-from pyfits.tests import PyfitsTestCase
-from pyfits.tests.util import catch_warnings, ignore_warnings
+from ..verify import VerifyError
+from . import PyfitsTestCase
+from .util import catch_warnings, ignore_warnings
 
 from nose.tools import assert_raises
 
 
 class TestHDUListFunctions(PyfitsTestCase):
+    @ignore_warnings
     def test_update_name(self):
         hdul = fits.open(self.data('o4sp040b0_raw.fits'))
         hdul[4].update_ext_name('Jim', "added by Jim")
@@ -58,7 +60,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
     def test_create_from_multiple_primary(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/145
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/145
 
         Ensure that a validation error occurs when saving an HDUList containing
         multiple PrimaryHDUs.
@@ -432,7 +434,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         assert hdul[0].header['EXTEND'] == True
 
     def test_new_hdulist_extend_keyword(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/114
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/114
 
         Tests that adding a PrimaryHDU to a new HDUList object updates the
         EXTEND keyword on that HDU.
@@ -459,7 +461,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         assert ((old_data + 1) == hdul[1].data).all()
 
     def test_open_file_with_end_padding(self):
-        """Regression test for https://trac.assembla.com/pyfits/ticket/106
+        """Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/106
 
         Open files with end padding bytes.
         """
@@ -477,7 +479,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
     def test_open_file_with_bad_header_padding(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/136
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/136
 
         Open files with nulls for header block padding instead of spaces.
         """
@@ -505,7 +507,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
     def test_update_with_truncated_header(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/148
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/148
 
         Test that saving an update where the header is shorter than the
         original header doesn't leave a stump from the old header in the file.
@@ -565,7 +567,7 @@ class TestHDUListFunctions(PyfitsTestCase):
 
     def test_update_resized_header2(self):
         """
-        Regression test for https://trac.assembla.com/pyfits/ticket/150
+        Regression test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/150
 
         This is similar to test_update_resized_header, but specifically tests a
         case of multiple consecutive flush() calls on the same HDUList object,
@@ -615,23 +617,25 @@ class TestHDUListFunctions(PyfitsTestCase):
                 assert orig_info == hdul2.info(output=False)
                 for idx in range(len(hdul)):
                     assert hdul[idx].header == hdul2[idx].header
-                    if hdul[idx].data is None or hdul2[idx].data is None:
-                        assert hdul[idx].data == hdul2[idx].data
-                    elif (hdul[idx].data.dtype.fields and
-                          hdul2[idx].data.dtype.fields):
-                        # Compare tables
-                        for n in hdul[idx].data.names:
-                            c1 = hdul[idx].data[n]
-                            c2 = hdul2[idx].data[n]
-                            assert (c1 == c2).all()
-                    else:
-                        assert (hdul[idx].data == hdul2[idx].data).all()
+
+                    with ignore_warnings():
+                        if hdul[idx].data is None or hdul2[idx].data is None:
+                            assert hdul[idx].data == hdul2[idx].data
+                        elif (hdul[idx].data.dtype.fields and
+                              hdul2[idx].data.dtype.fields):
+                            # Compare tables
+                            for n in hdul[idx].data.names:
+                                c1 = hdul[idx].data[n]
+                                c2 = hdul2[idx].data[n]
+                                assert (c1 == c2).all()
+                        else:
+                            assert (hdul[idx].data == hdul2[idx].data).all()
 
         for filename in glob.glob(os.path.join(self.data_dir, '*.fits')):
             if sys.platform == 'win32' and filename == 'zerowidth.fits':
                 # Running this test on this file causes a crash in some
                 # versions of Numpy on Windows.  See PyFITS ticket
-                # https://trac.assembla.com/pyfits/ticket/174
+                # https://aeon.stsci.edu/ssb/trac/pyfits/ticket/174
                 continue
             test_fromstring(os.path.join(self.data_dir, filename))
 
@@ -639,7 +643,7 @@ class TestHDUListFunctions(PyfitsTestCase):
         assert_raises(TypeError, fits.HDUList.fromstring, ['a', 'b', 'c'])
 
     def test_save_backup(self):
-        """Test for https://trac.assembla.com/pyfits/ticket/121
+        """Test for https://aeon.stsci.edu/ssb/trac/pyfits/ticket/121
 
         Save backup of file before flushing changes.
         """
@@ -725,9 +729,9 @@ class TestHDUListFunctions(PyfitsTestCase):
         def test(mmap_a, mmap_b):
             col_a = fits.Column(name='a', format='J', array=arr_a)
             col_b = fits.Column(name='b', format='J', array=arr_b)
-            hdu_a = fits.new_table([col_a])
+            hdu_a = fits.BinTableHDU.from_columns([col_a])
             hdu_a.writeto(self.temp('test_a.fits'), clobber=True)
-            hdu_b = fits.new_table([col_b])
+            hdu_b = fits.BinTableHDU.from_columns([col_b])
             hdu_b.writeto(self.temp('test_b.fits'), clobber=True)
 
             hdul_a = fits.open(self.temp('test_a.fits'), mode='update',
